import cloudscraper
from .utils.errors import errors

scraper = cloudscraper.create_scraper()


class Currency:
    def __init__(self) -> None:
        pass

    @staticmethod
    def balance(auth: str) -> int:
        """Get user's balance from their auth token"""

        request = scraper.get("https://api.bloxflip.com/user", headers={
            "x-auth-token": auth
        }).json()
        if not "user" in list(request):
            raise errors.InvalidAuthorization("Invalid authorization provided.")

        return request["user"]["wallet"]

    @staticmethod
    def affiliate(auth: str) -> int:
        """Get the amount of currency available on their affiliate"""

        request = scraper.get("https://api.bloxflip.com/user/affiliates", headers={
            "x-auth-token": auth
        }).json()
        if not "affiliateMoneyAvailable" in list(request):
            raise errors.InvalidAuthorization("Invalid authorization provided.")

        return request["affiliateMoneyAvailable"]

    @staticmethod
    def claimAfiliate(auth: str, amount: int) -> dict:
        """Claim the currency insied the user's affiliate balance"""

        response = scraper.post("https://api.bloxflip.com/user/affiliates/claim",
                    headers={
                        "x-auth-token": auth
                    }, json={
                        "amount": str(amount)
                    }
                )

        if response.status_code == 200:
            return response.json()

        elif response.status_code == 429:
            raise errors.Ratelimited("Ratelimited, too many requests.")

        elif Currency.Affiliate(auth) < 100:
            raise errors.InsufficientFunds("Not enough funds to withdraw")

        else:
            raise errors.InvalidAuthorization("Invalid authorization provided.")

    @staticmethod
    def withdraw(auth: str, amount: int) -> dict:
        """Withdraw Bloxflip currency into Roblox"""

        response = scraper.post("https://api.bloxflip.com/user/withdrawTarget", headers={
                    "x-auth-token": auth
                }, json={
                    "amount": str(int(amount))
                }
            )

        if not response.status_code == 200:
            raise errors.GeneralError(
                "Either you're withdrawing more than your balance or your auth is not connected to a valid cookie.")

        return response.json()



class errors:
    def __init__() -> None:
        pass

    class InsufficientFunds(Exception):
        """Raised if user lacks the funds to do a certain action"""

        pass

    class InvalidAuthorization(Exception):
        """Raised if an invalid Authorization Token is passed"""

        pass

    class GameAlreadyStarted(Exception):
        """Raised if there is an already active game"""

        pass

    class GameNotStarted(Exception):
        """Raised if there is no active game"""

        pass

    class Ratelimited(Exception):
        """Raised if user is ratelimited"""

        pass

    class InvalidParameter(Exception):
        """Raised if an invalid parameter is passed"""

        pass

    class GeneralError(Exception):
        """Raised on a general failures"""

        pass

    class NetworkError(Exception):
        """Raised on cloudflare or other network errors"""

        pass



import websocket, base64
from websocket import create_connection
from random import randbytes


def Websocket(headers: dict = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:103.0) Gecko/20100101 Firefox/103.0",
    "Accept": "*/*",
    "Accept-Language": "en-US,en;q=0.5",
    "Accept-Encoding": "gzip, deflate, br",
    "Sec-WebSocket-Version": "13",
    "Origin": "https://www.piesocket.com",
    "Sec-WebSocket-Extensions": "permessage-deflate",
    "Sec-WebSocket-Key": str(base64.b64encode(randbytes(16)).decode('utf-8')),
    "Connection": "keep-alive, Upgrade",
    "Sec-Fetch-Dest": "websocket",
    "Sec-Fetch-Mode": "websocket",
    "Sec-Fetch-Site": "cross-site",
    "Pragma": "no-cache",
    "Cache-Control": "no-cache",
    "Upgrade": "websocket"
}) -> websocket.WebSocket:
    """Connects to websocket and returns websocket object

    Parameters:
    headers (dict): Headers to be used to connect to websocket (Optional)

    Returns:
    websocket.WebSocket: A websocket connection connected and already logged in
    """

    return create_connection(
        "wss://ws.bloxflip.com/socket.io/?EIO=3&transport=websocket",
        suppress_origin=True,
        header=headers
    )



import cloudscraper, websocket, json, time
from .authorization import Authorization
from .websockets import Websocket
from .utils.errors import errors

scraper = cloudscraper.create_scraper()


class Round:
    """A wrapper for a Crash game"""

    def __init__(self, game: dict) -> None:
        self.crashpoint = game["crashPoint"]
        self.public_seed = game["publicSeed"]
        self.private_seed = game["privateSeed"]
        self.private_hash = game["privateHash"]
        self.game_id = game["_id"]
        self.players = [Authorization.lookup(player["playerID"], player["betAmount"]) for player in game["players"]]


class Crash:
    def __init__(self, auth: str) -> None:
        self.auth = auth

    class _Websocket:
        def __init__(self, auth: str) -> None:
            self.auth = auth
            self._connection = None

        def connect(self, headers: dict = None) -> websocket.WebSocket:
            """Connects to websocket and returns websocket object
            
            Parameters:
            headers (dict): Headers to be used to connect to websocket (Optional)

            Returns:
            websocket.WebSocket: A websocket connection connected and already logged in
            """

            self._connection = Websocket(headers) if headers else Websocket()

            ws = self._connection
            ws.send("40/crash,")
            ws.send(f'42/crash,["auth","{self.auth}"]')

            return self._connection

        @property
        def connection(self) -> websocket.WebSocket:
            return self._connection

        def join(self, betamount: float, multiplier: float) -> None:
            """Joins Crash game with the betamount as well as multiplier"""

            json = str(
                {
                    "autoCashoutPoint": int(multiplier * 100),
                    "betAmount": betamount
                }
            ).replace("'", '"').replace(" ", "")
            self._connection.send(f'42/crash,["join-game",{str(json)}]')

    def Websocket(self):
        return self._Websocket(self.auth)

    @staticmethod
    def crashpoints(amount: int = 35, interval: float = 0.01, on_game_start: type(print) = None) -> list:
        """Indefinitely yields the last game's results as well as the previous results everytime a new game starts
        
        Parameters:
        amount (int): Amount of games to return each time
        interval (float): Time to wait in between each api request

        Returns:
        list: Recent games
        
        """

        history = None
        sent = False

        if amount > 35:
            raise errors.InvalidParameter("Amount cannot be above 35.")

        elif not callable(on_game_start) and on_game_start:
            raise errors.InvalidParameter("'on_game_start' must be a callable object.")

        while True:
            try:
                games = scraper.get("https://api.bloxflip.com/games/crash").json()
            except ValueError:
                continue

            if history != games["history"]:
                history = games["history"]

                data = [Round(_crashpoint) for _crashpoint in history]

                if not sent:
                    sent = True
                    continue

                if on_game_start:
                    on_game_start(data)
                yield data

            time.sleep(interval)

    @property
    def history(self, amount: int = 35) -> list:
        """Returns the last N games, 35 by default
        
        Parameter:
        amount (int): Amount of games returned
        
        Returns:
        list: Recent games"""

        try:
            games = scraper.get("https://api.bloxflip.com/games/crash").json()
        except json.decoder.JSONDecodeError:
            raise errors.NetworkError("A Network Error has occurred.")

        history = games["history"]

        return [[Round(_crashpoint) for _crashpoint in history[:amount]]]



import cloudscraper, websocket, json, time
from .authorization import Authorization
from .websockets import Websocket
from .utils.errors import errors

scraper = cloudscraper.create_scraper()


class Game:
    def __init__(self, info):
        self.value = sum([player["betAmount"] for player in info["players"]])
        self.time = info["timeLeft"]
        self.status = info["status"]
        self.winner = info["winner"]
        self.winningColor = info["winningColor"]
        self.id = info["_id"]
        self.players = [Authorization.lookup(player["_id"], player["betAmount"]) for player in info["players"]]


class Jackpot:
    def __init__(self, auth: str) -> None:
        self.auth = auth

    class _Websocket:
        def __init__(self, auth: str) -> None:
            self.auth = auth
            self._connection = None

        def connect(self, headers: dict = None) -> websocket.WebSocket:
            """Connects to websocket and returns websocket object

            Parameters:
            headers (dict): Headers to be used to connect to websocket (Optional)

            Returns:
            websocket.WebSocket: A websocket connection connected and already logged in
            """

            self._connection = Websocket(headers) if headers else (Websocket())

            ws = self._connection
            ws.send("40/jackpot,")
            ws.send(f'42/jackpot,["auth","{self.auth}"]')

            return self._connection

        @property
        def connection(self) -> websocket.WebSocket:
            return self._connection

        def join(self, betamount: float, multiplier: float) -> None:
            """Joins Crash game with the betamount as well as multiplier"""

            json = str(
                {
                    "betAmount": betamount
                }
            ).replace("'", '"').replace(" ", "")
            self._connection.send(f'42/jackpot,["join-game",{str(json)}]')

    def Websocket(self):
        return self._Websocket(self.auth)

    @staticmethod
    def sniper(snipe_at: int = 0.05, interval: float = 0.01, on_game_start: type(print) = None) -> float:
        """Indefinitely yields the pot's value N seconds before wheel spins

        Parameters:
        amount (int): Amount of games to return each time
        interval (float): Time to wait in between each api request

        Returns:
        list: Recent games

        """

        if snipe_at > 29:
            raise errors.InvalidParameter("'snipe_at' cannot be above greater than 29")

        elif not callable(on_game_start) and on_game_start:
            raise errors.InvalidParameter("'on_game_start' must be a callable object.")

        while True:
            try:
                current = scraper.get("https://api.bloxflip.com/games/jackpot").json()["current"]
            except Exception as e:
                pass

            if len(current["players"]) == 2:
                time.sleep(30 - snipe_at)

                current = scraper.get("https://api.bloxflip.com/games/jackpot").json()["current"]

                pot_value = sum([player["betAmount"] for player in current["players"]])

                if on_game_start: on_game_start(pot_value)

                yield Game(current)

            time.sleep(interval)

    @property
    def current(self) -> float:
        """Returns the current game's pot value

        Returns:
        int: Current game's value
        """

        try:
            current = scraper.get("https://api.bloxflip.com/games/jackpot").json()["current"]
        except json.decoder.JSONDecodeError:
            raise errors.NetworkError("A Network Error has occurred")

        return Game(current)



import cloudscraper, json
from .utils.errors import errors

scraper = cloudscraper.create_scraper()

class Account:
    """An object for a user's account info"""

    def __init__(self, info: dict) -> None:
        self.games_played = info["gamesPlayed"]
        self.games_won = info["gamesWon"]
        self.account_verified = info["hasVerifiedAccount"]
        self.total_deposited = info["totalDeposited"]
        self.total_withdrawn = info["totalWithdrawn"]
        self.total_wagered = info["wager"]
        self.username = info["user"]["robloxUsername"]
        self.id = info["user"]["robloxId"]
        self.rank = info["user"]["rank"]
        self.balance = info["user"]["wallet"]

class User:
    """A class for the lookup info on somebodies account"""

    def __init__(self, info: dict, user_id: str, bet_amount: int) -> None:
        self.rank = info["rank"]
        self.username = info["username"]
        self.wagered = info["total_wagered"]
        self.games_played = info["gamesPlayed"]
        self.rain_winnings = info["rainWinnings"]
        self.trivia_winnings = info["triviaWinnings"]
        if bet_amount: self.bet_amount = bet_amount
        self.id = user_id


class Authorization:
    def __init__(self) -> None:
        pass

    @staticmethod
    def generate(cookie: str, affiliate: str = "BFSB") -> str:
        """Generate a Bloxflip Auth Token from a Roblox Cookie"""

        request = scraper.post("https://api.bloxflip.com/user/login", json={
            "affiliateCode": affiliate,
            "cookie": cookie
        }).json()

        if "jwt" in list(request):
            return request["jwt"]

        raise errors.GeneralError("Either cookie is invalid or cookie is ip locked.")

    @staticmethod
    def validate(auth: str) -> bool:
        """Validates that the Authorization Token works"""

        request = scraper.get("https://api.bloxflip.com/user", headers={
            "x-auth-token": auth
        }).json()

        if request["success"]:
            return True

        return False

    @staticmethod
    def get_info(auth: str) -> User:
        """Gets user's info then returns in a class"""

        try:
            request = scraper.get("https://api.bloxflip.com/user", headers={
                "x-auth-token": auth
            }).json()
        except json.decoder.JSONDecodeError:
            raise errors.NetworkError("Network Error.")

        try:
            return Account(request)
        except KeyError:
            raise errors.InvalidAuthorization("Invalid Authorization provided")

    @staticmethod
    def lookup(user_id: str, bet_amount: int = None) -> User:
        try:
            request = scraper.get(f"https://api.bloxflip.com/user/lookup/{user_id}").json()
        except json.decoder.JSONDecodeError:
            raise errors.NetworkError("Network Error.")

        if not request["success"]:
            raise errors.InvalidParameter("Invalid UserID")

        try:
            return User(request, str(user_id), bet_amount)
        except KeyError:
            raise errors.InvalidAuthorization("Invalid Authorization provided")



import cloudscraper, json, time
from .utils.errors import errors

scraper = cloudscraper.create_scraper()


class Tower:
    """A class for a Towers game"""

    def __init__(self, info: dict) -> None:
        if not info["hasGame"]:

            game = info["game"]
            self.competed_levels = info["completedLevels"]
            self.difficulty = info["difficulty"]
            self.exploded = info["exploded"]

            self.timestamp = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(info["created"]))
            self.client_seed = info["clientSeed"]
            self.user_id = info["userId"]
            self.nonce = info["nonce"]
            self.uuid = info["uuid"]

            self.bet_amount = info["betAmount"]
            self.payout = info["payout"]

            self.active = True
        else:
            self.active = False


class Towers:
    def __init__(self, auth):
        self.auth = auth

    def create(self, betamount: float, difficulty: str) -> dict:
        """Creates a towers game"""

        if not difficulty in ["easy", "normal", "hard"]:
            raise errors.InvalidParameter("Invalid difficulty provided.")

        response = scraper.post("https://rest-bf.blox.land/games/towers/create", headers={
                        "x-auth-token": self.auth
                    },
                    json={
                        "betAmount": str(betamount),
                        "difficulty": difficulty
                    }
                )

        if betamount < 5:
            raise errors.InvalidParameter("Bet amount must be greater than 5")

        if response.status_code == 429:
            raise errors.Ratelimited("Ratelimited: Too many requests")

        if not response.status_code == 200:
            try:
                response.json()
            except ValueError:
                raise errors.NetworkError("Network error.", "error")

            if response.json()["msg"] == "You already have an active towers game!":
                raise errors.GameAlreadyStarted("You already have an active towers game. End it then try again.")

            elif response.json()["msg"] == "You can not afford to start this game!":
                raise errors.InsufficientFunds("You cannot afford to start this game.")

            raise errors.GeneralError("Failed to create game")

        return response.json()

    def choose(self, choice: int) -> bool:
        """Chooses a tile to uncover"""

        response = scraper.post("https://rest-bf.blox.land/games/towers/action", headers={
                            "x-auth-token": self.auth
                        },
                        json={
                            "cashout": False,
                            "tile": choice
                        }
                    )

        if response.status_code == 429:
            raise errors.Ratelimited("Ratelimited: Too many requests")

        if not response.status_code == 200:
            if response.json()["msg"] == "You do not have an active towers game!":
                raise errors.GameNotStarted("There is currently no active towers game")

        return not response.json()["exploded"]

    def cashout(self) -> object:
        """Cashes out the winnings from the current game"""

        response = scraper.post("https://rest-bf.blox.land/games/towers/action", headers={
                            "x-auth-token": self.auth
                        },
                        json={
                            "cashout": True,

                        }
                    )

        if response.status_code == 429:
            raise errors.Ratelimited("Ratelimited: Too many requests")

        if not response.status_code == 200:
            if not "msg" in list(response.json()):
                raise errors.InvalidAuthorization("Invalid authorization provided.")

            elif response.json()["msg"] == "You do not have an active towers game!":
                raise errors.GameNotStarted("You do not have an active towers game.")

            elif response.json()["msg"] == "You cannot cash out yet! You must uncover at least one tile!":
                raise errors.GeneralError("You cannot cash out yet! You must uncover at least one tile!")

            raise errors.InsufficientFunds("Insuffecient funds")

        return response.json()

    @property
    def current(self) -> Tower:
        request = scraper.get("https://api.bloxflip.com/games/mines", headers={
                        "x-auth-token": self.auth
                    }
                ).json()

        try:
            return Tower(request)
        except (KeyError, Exception):
            raise errors.InvalidAuthorization("Invalid authorization provided.")



from setuptools import setup, find_packages

setup(
	name="Bloxflip.py",
	version = "2.2.19",
	author = "TheOnlyIceBear",
	description = "An api wrapper for https://bloxflip.com",
	packages=["bloxflip", "bloxflip/utils"]
)